/*
Copyright (C) Mohammad Mazarei  Zeus@Sisoog.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 3
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
02111-1307, USA.
*/

#include "stm32f10x.h"
#include "stm32f10x_dma.h"
#include "dac.h"
#include "misc.h"
#include "stm32f10x_tim.h"
#include "stm32f10x_gpio.h"
#include "stm32f10x_rcc.h"



void DAC_RCC_Configuration(void)
{   
  /* Enable peripheral clocks ------------------------------------------------*/
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA2, ENABLE);
  /* GPIOA Periph clock enable */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
  /* DAC Periph clock enable */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_DAC, ENABLE);
  /* TIM2 Periph clock enable */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);
	
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO,ENABLE);
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC,ENABLE);
  
}


void DAC_GPIO_Config(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  /* Once the DAC channel is enabled, the corresponding GPIO pin is automatically 
     connected to the DAC converter. In order to avoid parasitic consumption, 
     the GPIO pin should be configured in analog */
  GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_4 | GPIO_Pin_5;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
	
}


extern int bufferToProcess;
extern char dataRequest;
void DMA2_Channel4_5_IRQHandler(void){
  

  
   if(DMA_GetITStatus(DMA2_IT_HT4))	 /* LOW_EMPTY */
   {
     DMA_ClearITPendingBit(DMA2_IT_GL4|DMA2_IT_HT4);       
        audio_buffer_fill |= LOW_EMPTY;
   }  
   
   
   if(DMA_GetITStatus(DMA2_IT_TC4))  /* HIGH_EMPTY */
   {
      DMA_ClearITPendingBit(DMA2_IT_GL4|DMA2_IT_TC4); 

         audio_buffer_fill |= HIGH_EMPTY;
   }     
}



AUDIO_PlaybackBuffer_Status AUDIO_PlaybackBuffer_GetStatus(AUDIO_PlaybackBuffer_Status value)
{
    if ( value )
        audio_buffer_fill &= ~value;
    return audio_buffer_fill; 
}





void DAC_Set_Reat(int frq){

  TIM_TimeBaseInitTypeDef    TIM_TimeBaseStructure;
   /* TIM2 Configuration */
  TIM_TimeBaseStructInit(&TIM_TimeBaseStructure);
  TIM_TimeBaseStructure.TIM_Prescaler =  72; // 72 For 1us resolution; 
  TIM_TimeBaseStructure.TIM_Period = 1000000 / frq;          
        
  TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;    
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up; 
  TIM_TimeBaseStructure.TIM_RepetitionCounter = 0;
  TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);

  /* TIM2 TRGO selection */
  TIM_SelectOutputTrigger(TIM2, TIM_TRGOSource_Update);  
}



void DAC_START(void){
  TIM_Cmd(TIM2, ENABLE);
}


void DAC_STOP(void){
	
	  //DAC_Cmd(DAC_Channel_1, DISABLE);
		//DAC_Cmd(DAC_Channel_2, DISABLE);
	DMA_Cmd(DMA2_Channel4, DISABLE);                    /* Disable DMA Channel for Audio */
  TIM_Cmd(TIM2, DISABLE);
}


void DAC_PAUSE(void){
  TIM_Cmd(TIM2, DISABLE);
}


void DAC_DMA_INIT(uint32_t *buffer,int size,char num_ch,char BitsPerSample){
 DMA_InitTypeDef            DMA_InitStructure;
    NVIC_InitTypeDef NVIC_InitStructure;
    
   
       /* DMA IRQ Channel configuration */
   NVIC_InitStructure.NVIC_IRQChannel = DMA2_Channel4_5_IRQn;
   NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
   NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
   NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
   NVIC_Init(&NVIC_InitStructure);
    
  /* DMA2 channel4 configuration */
  DMA_DeInit(DMA2_Channel4);


      DMA_InitStructure.DMA_MemoryBaseAddr =      (uint32_t) buffer;
//      DMA_InitStructure.DMA_BufferSize =          size;
//      DMA_InitStructure.DMA_PeripheralBaseAddr =  (uint32_t) &DAC->DHR12L1;    // 12 Bit
//      DMA_InitStructure.DMA_PeripheralDataSize =  DMA_PeripheralDataSize_HalfWord;
//      DMA_InitStructure.DMA_MemoryDataSize =      DMA_MemoryDataSize_HalfWord;
  
  
  if(BitsPerSample==8){   // For Wave File
      if(num_ch==1){  // mono
      DMA_InitStructure.DMA_BufferSize =         size;
      DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t) &DAC->DHR8R1;   // 8Bit Reg
      DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
      DMA_InitStructure.DMA_MemoryDataSize =     DMA_MemoryDataSize_Byte;  
      
      }else{        // Storio
      DMA_InitStructure.DMA_BufferSize =          size/2;
      DMA_InitStructure.DMA_PeripheralBaseAddr =  (uint32_t) &DAC->DHR8RD;
      DMA_InitStructure.DMA_PeripheralDataSize =  DMA_PeripheralDataSize_HalfWord;
      DMA_InitStructure.DMA_MemoryDataSize =      DMA_MemoryDataSize_HalfWord;
      }
  }else{
      if(num_ch==1){  // mono
      DMA_InitStructure.DMA_BufferSize =         size/2;
      DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t) &DAC->DHR12L1;   // 12Bit Reg
      DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
      DMA_InitStructure.DMA_MemoryDataSize =     DMA_MemoryDataSize_HalfWord;  
      
      }else{        // Storio
      DMA_InitStructure.DMA_BufferSize =          size/4;
      DMA_InitStructure.DMA_PeripheralBaseAddr =  (uint32_t) &DAC->DHR12LD;    // 12 Bit
      DMA_InitStructure.DMA_PeripheralDataSize =  DMA_PeripheralDataSize_Word;
      DMA_InitStructure.DMA_MemoryDataSize =      DMA_MemoryDataSize_Word;
      }
  }
  
  DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
  DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
  DMA_InitStructure.DMA_Priority = DMA_Priority_High;
  DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;

  
  DMA_Init(DMA2_Channel4, &DMA_InitStructure);
  /* Enable DMA2 Channel4 */
  DMA_Cmd(DMA2_Channel4, ENABLE);   
  DMA_ITConfig(DMA2_Channel4, DMA_IT_TC | DMA_IT_HT, ENABLE); 
  /* Enable DMA for DAC Channel2 */
  DAC_DMACmd(DAC_Channel_2, ENABLE);
}







void init_dac(void){
  DAC_InitTypeDef            DAC_InitStructure;
  
  DAC_RCC_Configuration();
  DAC_GPIO_Config();
  
  
  
  DAC_StructInit(&DAC_InitStructure);
    /* DAC channel1 Configuration */
  DAC_InitStructure.DAC_Trigger = DAC_Trigger_T2_TRGO;
  DAC_InitStructure.DAC_WaveGeneration = DAC_WaveGeneration_None;
  //DAC_InitStructure.DAC_OutputBuffer = DAC_OutputBuffer_Disable;
  DAC_InitStructure.DAC_OutputBuffer = DAC_OutputBuffer_Enable;
  
  DAC_Init(DAC_Channel_1, &DAC_InitStructure);
  /* DAC channel2 Configuration */
  DAC_Init(DAC_Channel_2, &DAC_InitStructure);
  /* Enable DAC Channel1: Once the DAC channel1 is enabled, PA.04 is 
     automatically connected to the DAC converter. */
  DAC_Cmd(DAC_Channel_1, ENABLE);
  /* Enable DAC Channel2: Once the DAC channel2 is enabled, PA.05 is 
     automatically connected to the DAC converter. */
  DAC_Cmd(DAC_Channel_2, ENABLE);
}

