/*
Copyright (C) Mohammad Mazarei  Zeus@Sisoog.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 3
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
02111-1307, USA.
*/

#include "stm32f10x.h"
#include "mp3common.h"
#include <string.h>
#include <stdio.h>
#include "dac.h"
#include "ff.h"
#include "stm32f10x_dma.h"
#include "stm32f10x_iwdg.h"

char LCD_BUF[30];
#define LCD_STR(x,y,str)      lcd_gotoxy(x,y);lcd_str(str,0)

/* Type used to read out ID3 Tag, this type fits to the ID3V1 Format */
typedef struct
{
    char tag[3];
    char title[30];
    char artist[30];
    char album[30];
    char year[4];
    char comment[30];
    uint8_t genre;
} tID3V3;
/******************************* File ********************************/
/* Private define ------------------------------------------------------------*/
#define READBUF_SIZE  (5 * 1024)       /* Value must min be 2xMAINBUF_SIZE = 2x1940 = 3880bytes */
/* Private variables ---------------------------------------------------------*/
AUDIO_PlaybackBuffer_Status BufferStatus; /* Status of buffer */
s16 AudioBuffer[AudioBuffer_Size];          /* Playback buffer - Value must be 4608 to hold 2xMp3decoded frames */
MP3FrameInfo mp3FrameInfo;      /* Content is the output from MP3GetLastFrameInfo, 
                                   we only read this once, and conclude it will be the same in all frames
                                   Maybe this needs to be changed, for different requirements */

static HMP3Decoder hMP3Decoder; /* Content is the pointers to all buffers and information for the MP3 Library */
volatile uint32_t bytesLeft;    /* Saves how many bytes left in readbuf	*/
volatile uint32_t outOfData;    /* Used to set when out of data to quit playback */
volatile AUDIO_PlaybackBuffer_Status  audio_buffer_fill ; 
uint8_t readBuf[READBUF_SIZE];  /* Read buffer where data from SD card is read to */ 
uint8_t *readPtr;               /* Pointer to the next new data */
uint32_t n_Read;                /* File R/W count */



void AUDIO_Init_audio_mode(AUDIO_Length_enum length, uint16_t frequency, AUDIO_Format_enum format);



void AUDIO_Playback_Stop(void)
{
   
   DAC_STOP();
}



void Read_ID3V1(FIL *mp3_File)
    {
    tID3V3 tst_buff;
    uint8_t i;
    f_lseek  (mp3_File, (mp3_File->fsize) - sizeof(tst_buff) ); 
    f_read (mp3_File, &tst_buff,sizeof(tst_buff),(uint32_t *)&i); 
    f_lseek  (mp3_File,0);       
    
//    if ( strstr(tst_buff.tag, "TAG") )
//    {
// 	   printf("-- title %s   \r\n",tst_buff.title);	 
// 	   printf("-- artist %s  \r\n",tst_buff.artist);	 
// 	   printf("-- album %s   \r\n",tst_buff.album); 
//	   printf("-- year %s    \r\n",tst_buff.year);
//	   printf("-- comment %s \r\n",tst_buff.comment);	 	 
//    }
}


const char* MP3_GetStrResult(int err);

/*******************************************************************************/
// Function Name  : FillBuffer
// Description    : Fill the buffer with Music from MP3 Data
// Input          : - FileObject: file system 
//                  - Start: If 1, we will fill buffer nomatter the status of the playback,
//                           used in beginning of file (pre-fill)
// Output         : None
// Return         : None
// Attention		 : None
//******************************************************************************
int FillBuffer(FIL *mp3_File,uint8_t start)  
{
    uint8_t word_Allignment;   /* Variable used to make sure DMA transfer is alligned to 32bit */                
    int err=0;                 /* Return value from MP3decoder */
    int offset;                /* Used to save the offset to the next frame */
    int j;
	
    if(!start)    /* If we are in start pos we overwrite bufferstatus */ 
        BufferStatus = AUDIO_PlaybackBuffer_GetStatus( (AUDIO_PlaybackBuffer_Status) 0 ); 
    else
        BufferStatus = (AUDIO_PlaybackBuffer_Status) ( LOW_EMPTY | HIGH_EMPTY );    

        /* somewhat arbitrary trigger to refill buffer - should always be enough for a full frame */  
		if (bytesLeft < 2*MAINBUF_SIZE /*&& !eofReached*/) 
		{ 
          /* move last, small chunk from end of buffer to start, then fill with new data */ 
        if(mp3_File->fptr != mp3_File->fsize){
          word_Allignment = 4 - (bytesLeft % 4 );   /* Make sure the first byte in writing pos. is word alligned */ 
          memmove(readBuf+word_Allignment, readPtr, bytesLeft);
          f_read(mp3_File,(uint8_t *)readBuf+word_Allignment + bytesLeft,READBUF_SIZE - bytesLeft - word_Allignment,&n_Read);
      }else{
          n_Read = 0;
        }
		 
		  if( n_Read !=0 )
		  {
             /* zero-pad to avoid finding false sync word after last frame (from old data in readBuf) */
                       if (n_Read < READBUF_SIZE - bytesLeft - word_Allignment)
                       {    
		                  memset(readBuf + bytesLeft + n_Read, 0, READBUF_SIZE - bytesLeft - n_Read);
              	        }	          
                    bytesLeft += n_Read;
		              readPtr = readBuf+word_Allignment;
		  }	
		  else if(bytesLeft==0)  
		  {
		     outOfData = 1;
		  }
        
		}  

        if(outOfData == 1)
        {
            /* Stop Playback */  
            AUDIO_Playback_Stop();
            return 0; 
        }

		/* find start of next MP3 frame - assume EOF if no sync found */
		offset = MP3FindSyncWord(readPtr, bytesLeft);   
		if (offset < 0) 
		{
		    bytesLeft = 0;
		    return -1;
		}
    
		readPtr += offset;			  /* data start point */
		bytesLeft -= offset;		  /* in buffer */
		
                if (BufferStatus & LOW_EMPTY) /* Decode data to low part of buffer */
                {
                   err = MP3Decode(hMP3Decoder, &readPtr, (int *)&bytesLeft, (short *)AudioBuffer, 0); 	
  		             BufferStatus = AUDIO_PlaybackBuffer_GetStatus(LOW_EMPTY);				
                   
                   for(j=0;j<mp3FrameInfo.outputSamps;j++)
                        AudioBuffer[j]=(unsigned short)AudioBuffer[j] ^ (unsigned short)0x8000;
                }
 		else if (BufferStatus & HIGH_EMPTY)     /* Decode data to the high part of buffer */
                {
		   err = MP3Decode(hMP3Decoder, &readPtr, (int *)&bytesLeft, (short *)AudioBuffer+mp3FrameInfo.outputSamps,0);	
		   BufferStatus = AUDIO_PlaybackBuffer_GetStatus(HIGH_EMPTY);	 			 
                   
                   for(j=0;j<mp3FrameInfo.outputSamps;j++)
                        AudioBuffer[j+mp3FrameInfo.outputSamps]=(unsigned short)AudioBuffer[j+mp3FrameInfo.outputSamps] ^ (unsigned short)0x8000;
                } 
                
                
                
        if (err!=0) 
        {  
            bytesLeft=0;
            readPtr = readBuf;
//            printf("-- MP3Decode Error Occurred %s \r\n",MP3_GetStrResult(err));
            /* error occurred */
            switch (err) /* There is not implemeted any error handling. it will quit the playing in case of error */
            {
                /* do nothing - next call to decode will provide more mainData */
                default:
				  return -1; 				  
            }
        }     
  return 0;
}




const char* MP3_GetStrResult(int err)
{
    switch(err) {
        case ERR_MP3_NONE :                   return "ERR_MP3_NONE";
        case ERR_MP3_INDATA_UNDERFLOW :       return "ERR_MP3_INDATA_UNDERFLOW";
        case ERR_MP3_MAINDATA_UNDERFLOW :     return "ERR_MP3_MAINDATA_UNDERFLOW";
        case ERR_MP3_FREE_BITRATE_SYNC :      return "ERR_MP3_FREE_BITRATE_SYNC";
        case ERR_MP3_OUT_OF_MEMORY :          return "ERR_MP3_OUT_OF_MEMORY";
        case ERR_MP3_NULL_POINTER :           return "ERR_MP3_NULL_POINTER";
        case ERR_MP3_INVALID_FRAMEHEADER :    return "ERR_MP3_INVALID_FRAMEHEADER";   
        case ERR_MP3_INVALID_SIDEINFO :       return "ERR_MP3_INVALID_SIDEINFO"; 
        case ERR_MP3_INVALID_SCALEFACT :      return "ERR_MP3_INVALID_SCALEFACT";
        case ERR_MP3_INVALID_HUFFCODES :      return "ERR_MP3_INVALID_HUFFCODES";
        case ERR_MP3_INVALID_DEQUANTIZE :     return "ERR_MP3_INVALID_DEQUANTIZE";
        case ERR_MP3_INVALID_IMDCT :          return "ERR_MP3_INVALID_IMDCT";
        case ERR_MP3_INVALID_SUBBAND :        return "ERR_MP3_INVALID_SUBBAND";
        case ERR_UNKNOWN :                    return "ERR_UNKNOWN";  
        default :                             return "?";
    }  
}







int  MP3_PlayAudioFile(char *File_name)
{
   FIL fsrc; 
   /* Reset counters */
   bytesLeft = 0;   
   outOfData = 0;  
	IWDG_ReloadCounter();	
   printf("Play:%s",File_name);
   if(f_open(&fsrc, File_name, FA_OPEN_EXISTING | FA_READ)) { 
		 printf(" --Faild\r\n");
		 return 1;
	 }
	 IWDG_ReloadCounter();
   readPtr = readBuf;  
   hMP3Decoder = MP3InitDecoder();	 
   Read_ID3V1(&fsrc);  
   
   /* Read the first data to get info about the MP3 File */
   while( FillBuffer(&fsrc, 1 ) ) 	 /* Must Read MP3 file header information */
			IWDG_ReloadCounter();
	 
   /* Get the length of the decoded data, so we can set correct play length */                        
   MP3GetLastFrameInfo(hMP3Decoder, &mp3FrameInfo); 
  
   /* Select the correct samplerate and Mono/Stereo */
   AUDIO_Init_audio_mode( ( AUDIO_Length_enum )mp3FrameInfo.bitsPerSample,  \
			 ((MP3DecInfo *)hMP3Decoder)->samprate,  \
			 (((MP3DecInfo *)hMP3Decoder)->nChans==1) ? MONO : STEREO);
	 IWDG_ReloadCounter();
   /* Start the playback */ 
   //__enable_interrupt();	
   FillBuffer(&fsrc,0); // Load Low Buffer 
   FillBuffer(&fsrc,0); // Load High Buffer
	 IWDG_ReloadCounter();
   DAC_START();   			// Start DAC
   while( !(outOfData==1) )
   {
      IWDG_ReloadCounter();
		 FillBuffer(&fsrc,0);
   }
		DAC_STOP();
		f_close(&fsrc);
	 printf(" --Ok\r\n");
	 return 0;
}





void AUDIO_Init_audio_mode(AUDIO_Length_enum length, uint16_t frequency, AUDIO_Format_enum format)
{

    init_dac(); 
    DAC_Set_Reat(frequency);

   /* Buffers are supposed to be empty here	*/
   audio_buffer_fill = (AUDIO_PlaybackBuffer_Status) (LOW_EMPTY | HIGH_EMPTY );
   
   DAC_DMA_INIT((uint32_t*)((s8 *)AudioBuffer),
                (mp3FrameInfo.outputSamps*2)*2  // Dual Buffer in 'Int' Convert To Byte Size
                ,format+1,                      // nChans -> Storio Or Mono
                length);                        // Bits Per Sample
   
//LCD_STR(0,0,"MP3 file header");
//sprintf(LCD_BUF,"bitrate:%i%c",mp3FrameInfo.bitrate,0);
//LCD_STR(0,1,LCD_BUF);
//sprintf(LCD_BUF,"samprate:%i%c",frequency,0);
//LCD_STR(0,2,LCD_BUF);
//sprintf(LCD_BUF,"layer:%i%c",mp3FrameInfo.layer,0);
//LCD_STR(0,3,LCD_BUF);
//sprintf(LCD_BUF,"version:%i%c",mp3FrameInfo.version,0);
//LCD_STR(0,4,LCD_BUF);
//if(format==0){
//  LCD_STR(0,5,"nChans: Mono");
//}else{
//  LCD_STR(0,5,"nChans: Stereo");
//}
//
//sprintf(LCD_BUF,"bitsPerSample:%i%c",length,0);
//LCD_STR(0,6,LCD_BUF);
//   printf( "-- MP3 file header information \r\n");
//   printf( "-------------------------------- \r\n");
//   printf( "-- MP3 bitrate        = %d\r\n", mp3FrameInfo.bitrate);	   /* ?? */
//   printf( "-- MP3 bitsPerSample  = %d\r\n", length);
//   printf( "-- MP3 samprate       = %d\r\n", frequency);  
//   printf( "-- MP3 nChans         = %d\r\n", format);  
//   printf( "-- MP3 nGranSamps     = %d\r\n", ((MP3DecInfo *)hMP3Decoder)->nGranSamps);
//   /* ????? FrameSize = (((MpegVersion == MPEG1 ? 144 : 72) * Bitrate) / SamplingRate) + PaddingBit */
//   printf( "-- MP3 outputSamps    = %d\r\n", mp3FrameInfo.outputSamps);  
//   printf( "-- MP3 layer          = %d\r\n", mp3FrameInfo.layer);
//   printf( "-- MP3 version        = %d\r\n", mp3FrameInfo.version);
}





















