/*
Copyright (C) Mohammad Mazarei  Zeus@Sisoog.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 3
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
02111-1307, USA.
*/

#include "stm32f10x.h"
#include "ff.h"
#include <stdio.h>
#include "dac.h"
#include "stm32f10x_dma.h"
#include "core_cm3.h"
#include "stm32f10x_iwdg.h"

#define READ_BUFF_SIZE	256

#define FCC(c1,c2,c3,c4)	(((DWORD)c4<<24)+((DWORD)c3<<16)+((WORD)c2<<8)+(BYTE)c1)	/* FourCC */
#define fmt_chunk	FCC('f','m','t',' ')
#define data_chunk	FCC('d','a','t','a')
#define LIST_chunk	FCC('L','I','S','T')
#define fact_chunk	FCC('f','a','c','t')

BYTE Buff[READ_BUFF_SIZE];
WORD NumChannels, BitsPerSample;

extern FATFS Fs;			/* File system object */

typedef struct _wave_info{
  char NumChannels;
  char BitsPerSample;
  int  SamplingRate;
}wave_info;

#define  DAC_BUFFER_SIZE  AudioBuffer_Size*2
//char dacBuffer[DAC_BUFFER_SIZE];
#define dacBuffer       ((char *)AudioBuffer)
wave_info fw;

DWORD load_header (FIL *fsrc,wave_info *Wave);	/* 0:Invalid format, 1:I/O error, >1:Length in Sec */





UINT fill_wave_data(FIL *_fsrc){
  UINT  dataRead=1;
  int i;
	IWDG_ReloadCounter();
	
            if(audio_buffer_fill&LOW_EMPTY){
            audio_buffer_fill &= ~LOW_EMPTY;
            f_read(_fsrc,(char*)dacBuffer,DAC_BUFFER_SIZE/2,&dataRead);
            	if(fw.BitsPerSample==16)
                   for(i=1;i<DAC_BUFFER_SIZE/2;i+=2)
                         *(dacBuffer+i) = *(dacBuffer+i)^0x80;
                        
          }else if(audio_buffer_fill&HIGH_EMPTY)
		{
                        audio_buffer_fill &= ~HIGH_EMPTY;
			f_read(_fsrc,(char *)(dacBuffer+(DAC_BUFFER_SIZE/2)),DAC_BUFFER_SIZE/2,&dataRead);
			if(fw.BitsPerSample==16)
                            for(i=1;i<DAC_BUFFER_SIZE/2;i+=2)
                                 *(dacBuffer+(DAC_BUFFER_SIZE/2)+i) = *(dacBuffer+(DAC_BUFFER_SIZE/2)+i)^0x80; 
		}
   IWDG_ReloadCounter();
   return dataRead; 
}




void play_wave(char *file_name){
  FIL fsrc;      /* file objects */
  BYTE res;
  
	printf("Try Play:%s",file_name);
  res = f_open(&fsrc, file_name, FA_OPEN_EXISTING | FA_READ);
  if(res) {
		printf(" --Faild\r\n");
		return;
	}
	IWDG_ReloadCounter();
  res = load_header(&fsrc,&fw); // Get Wave File Info
  if(res<2) 
	{
		printf(" --Faild\r\n");		
	 return;
  }
  
  /********************************** Play File *****************************/

IWDG_ReloadCounter();
       audio_buffer_fill = (AUDIO_PlaybackBuffer_Status) (LOW_EMPTY | HIGH_EMPTY );
       fill_wave_data(&fsrc); // Load Buffer L
       fill_wave_data(&fsrc); // Load Buffer H
       
      //init_dac();
      DAC_DMA_INIT((uint32_t*)dacBuffer,DAC_BUFFER_SIZE,fw.NumChannels,fw.BitsPerSample);
      DAC_Set_Reat(fw.SamplingRate);
      //__enable_interrupt();	
      DAC_START();
IWDG_ReloadCounter();   
  
	for(;;)
	{
         
      if(fill_wave_data(&fsrc)==0)
			{
			//__disable_interrupt();
			DAC_STOP();
      f_close(&fsrc); 
			printf(" --Ok\r\n");
			return ;
			}
			IWDG_ReloadCounter();
	} 
  
  
  
  
   
}








DWORD load_header (FIL *fsrc,wave_info *Wave)	/* 0:Invalid format, 1:I/O error, >1:Length in Sec */
{
	DWORD ChunkID, SamplingRate, sz, Length;
	UINT rb;			/* Return value */
        
	if (f_read(fsrc,Buff, 12, &rb)) return 1;	/* Load file header (12 bytes) */
	if (rb != 12 || LD_DWORD(Buff+8) != FCC('W','A','V','E')) return 0;
	

	while(1) {
		f_read(fsrc,Buff, 8, &rb);			/* Get Chunk ID and size */
		if (rb != 8) return 0;
		ChunkID = LD_DWORD(&Buff[0]);
		sz = LD_DWORD(&Buff[4]);		/* Chunk size */	

		switch (ChunkID) {	/* FCC */
			case fmt_chunk:					        /* 'fmt ' chunk */
				if (sz > 100 || sz < 16) return 0;		/* Check chunk size */
				f_read(fsrc,Buff, sz, &rb);
                                if (rb != sz) return 0;
				if (Buff[0] != 1) return 0;				/* Check coding type (PCM=1) */

				NumChannels = Buff[2];					/* Get channel flag */
				if (NumChannels != 1 && NumChannels != 2) 		/* Check channels (1/2) */
					return 0;
				if(NumChannels == 1) 	        Wave->NumChannels =1;
				else if(NumChannels == 2) 	Wave->NumChannels =2;

				BitsPerSample = Buff[14];		/* Resolution flag */
				if (BitsPerSample != 8 && BitsPerSample != 16)	/* Check resolution (8/16) */
					return 0;
				if(BitsPerSample == 8) 	        Wave->BitsPerSample=8;
				else if(BitsPerSample == 16) 	Wave->BitsPerSample=16;

				SamplingRate = LD_DWORD(Buff+4);
                                Wave->SamplingRate = SamplingRate;
				//set_sampling_rate( SamplingRate );			/* Sampling freq */
				
				
				break;
	
			case data_chunk:				/* 'data' chunk (start to play) */
				Length = (sz*100) / (SamplingRate * NumChannels * (BitsPerSample>>3) );	// length in des seconds
				//sprintf(str, "%ld:%2ld min", (Length/60), (Length%60));
				return Length;
	
			case LIST_chunk:				/* 'LIST' chunk (skip) */

			case fact_chunk:				/* 'fact' chunk (skip) */
				f_lseek(fsrc, sz);
				break;
	
			default :								/* Unknown chunk (error) */
				return 0;
		}
	}
}

